<?php

namespace SGalinski\SgNews\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use SGalinski\SgNews\Domain\Model\News;
use SGalinski\SgNews\Service\HeaderMetaDataService;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * Controller that handles the list rendering of a single category
 */
class ListByCategoryController extends AbstractController {
	/**
	 * @inject
	 * @var \SGalinski\SgNews\Domain\Repository\CategoryRepository
	 */
	protected $categoryRepository;

	/**
	 * @inject
	 * @var \SGalinski\SgNews\Domain\Repository\TagRepository
	 */
	protected $tagRepository;

	/**
	 * @inject
	 * @var \SGalinski\SgNews\Domain\Repository\NewsRepository
	 */
	protected $newsRepository;

	/**
	 * Renders the news list of a category
	 *
	 * @param array $newsMetaData
	 * @return void
	 * @throws \TYPO3\CMS\Extbase\Persistence\Exception\InvalidQueryException
	 */
	public function indexAction(array $newsMetaData = []) {
		$filterByCategories = FALSE;
		$categoryUids = GeneralUtility::intExplode(',', $this->settings['categories']);
		$tagUids = GeneralUtility::intExplode(',', $this->settings['tags'], TRUE);
		foreach ($categoryUids as $categoryUid) {
			if ($categoryUid > 0) {
				$filterByCategories = TRUE;
				break;
			}
		}

		if (!$filterByCategories) {
			$categoryUids = [(int) $GLOBALS['TSFE']->id];
		}

		$categories = [];
		foreach ($categoryUids as $categoryUid) {
			$categories[$categoryUid] = $this->categoryRepository->findByUid($categoryUid);
		}

		$startTime = (int) $this->settings['starttime'];
		$endTime = (int) $this->settings['endtime'];

		$offset = 0;
		$newsPerPage = (int) $this->settings['newsLimitPerPage'];
		$currentPageBrowserPage = (int) GeneralUtility::_GP('tx_sgnews_pagebrowser')['currentPage'];
		if ($currentPageBrowserPage && $newsPerPage) {
			$offset = $currentPageBrowserPage * $newsPerPage;
		}
		$newsCount = $this->newsRepository->newsCountByCategories($categoryUids, $tagUids, $startTime, $endTime);
		$numberOfPages = ($newsPerPage <= 0 ? 0 : ceil($newsCount / $newsPerPage));

		$headerSet = FALSE;
		$news = $this->newsRepository->findAllSortedNewsByCategories(
			$categoryUids, $newsPerPage, $offset, $this->settings['sortBy'], $tagUids, $startTime, $endTime
		)->toArray();
		foreach ($news as $newsEntry) {
			/** @var News $newsEntry */
			$data = $this->getMetaDataForNews($newsEntry, $categories[$newsEntry->getPid()]);
			$newsMetaData[] = $data;

			if (!$headerSet) {
				if ($data['image']) {
					HeaderMetaDataService::addOgImageToHeader($data['image']);
					$headerSet = TRUE;
				} elseif ($data['teaserImage']) {
					HeaderMetaDataService::addOgImageToHeader($data['teaserImage']);
					$headerSet = TRUE;
				}
			}
		}

		// Redo this function, until one variable get the amount of $newsPerPage. Reduces the amount of ajax calls. Needed because of languagevisibility.
		if (count($newsMetaData) < $newsPerPage) {
			$nextPage = $currentPageBrowserPage + 1;
			if ($nextPage <= $numberOfPages) {
				GeneralUtility::_GETset(['tx_sgnews_pagebrowser' => ['currentPage' => $nextPage]]);
				$this->indexAction($newsMetaData);
				return;
			}
		}

		$this->view->assign('numberOfPages', $numberOfPages);
		$this->view->assign('newsMetaData', $newsMetaData);
		$this->view->assign('categories', $categories);
	}
}
